@@ -183,6 +183,8 @@ impl Engine {
     /// Register a custom type for use with the [`Engine`].
     /// The type must implement [`Clone`].
     ///
+    /// Not available under `no_object`.
+    ///
     /// # Example
     ///
     /// ```
@@ -224,6 +226,8 @@ impl Engine {
     /// Register a custom type for use with the [`Engine`], with a pretty-print name
     /// for the `type_of` function. The type must implement [`Clone`].
     ///
+    /// Not available under `no_object`.
+    ///
     /// # Example
     ///
     /// ```
@@ -265,7 +269,8 @@ impl Engine {
     #[inline(always)]
     pub fn register_type_with_name<T: Variant + Clone>(&mut self, name: &str) -> &mut Self {
         // Add the pretty-print type name into the map
-        self.type_names.insert(type_name::<T>().into(), name.into());
+        self.type_names
+            .insert(type_name::<T>().into(), Box::new(name.into()));
         self
     }
     /// Register an type iterator for an iterable type with the [`Engine`].
@@ -283,6 +288,8 @@ impl Engine {
     ///
     /// The function signature must start with `&mut self` and not `&self`.
     ///
+    /// Not available under `no_object`.
+    ///
     /// # Example
     ///
     /// ```
@@ -328,6 +335,8 @@ impl Engine {
     ///
     /// The function signature must start with `&mut self` and not `&self`.
     ///
+    /// Not available under `no_object`.
+    ///
     /// # Example
     ///
     /// ```
@@ -373,6 +382,8 @@ impl Engine {
     }
     /// Register a setter function for a member of a registered type with the [`Engine`].
     ///
+    /// Not available under `no_object`.
+    ///
     /// # Example
     ///
     /// ```
@@ -419,6 +430,8 @@ impl Engine {
     }
     /// Register a setter function for a member of a registered type with the [`Engine`].
     ///
+    /// Not available under `no_object`.
+    ///
     /// # Example
     ///
     /// ```
@@ -464,15 +477,15 @@ impl Engine {
         set_fn: impl Fn(&mut T, V) -> Result<(), Box<EvalAltResult>> + SendSync + 'static,
     ) -> &mut Self {
         use crate::engine::make_setter;
-        self.register_result_fn(&make_setter(name), move |obj: &mut T, value: V| {
-            set_fn(obj, value)
-        })
+        self.register_result_fn(&make_setter(name), set_fn)
     }
     /// Short-hand for registering both getter and setter functions
     /// of a registered type with the [`Engine`].
     ///
     /// All function signatures must start with `&mut self` and not `&self`.
     ///
+    /// Not available under `no_object`.
+    ///
     /// # Example
     ///
     /// ```
@@ -521,6 +534,8 @@ impl Engine {
     ///
     /// The function signature must start with `&mut self` and not `&self`.
     ///
+    /// Not available under both `no_index` and `no_object`.
+    ///
     /// # Panics
     ///
     /// Panics if the type is [`Array`], [`Map`], [`String`], [`ImmutableString`][crate::ImmutableString] or `&str`.
@@ -555,16 +570,18 @@ impl Engine {
     ///     // Register an indexer.
     ///     .register_indexer_get(TestStruct::get_field);
     ///
+    /// # #[cfg(not(feature = "no_index"))]
     /// assert_eq!(engine.eval::<i64>("let a = new_ts(); a[2]")?, 3);
     /// # Ok(())
     /// # }
     /// ```
-    #[cfg(not(feature = "no_index"))]
+    #[cfg(any(not(feature = "no_index"), not(feature = "no_object")))]
     #[inline(always)]
     pub fn register_indexer_get<T: Variant + Clone, X: Variant + Clone, V: Variant + Clone>(
         &mut self,
         get_fn: impl Fn(&mut T, X) -> V + SendSync + 'static,
     ) -> &mut Self {
+        #[cfg(not(feature = "no_index"))]
         if TypeId::of::<T>() == TypeId::of::<Array>() {
             panic!("Cannot register indexer for arrays.");
         }
@@ -585,6 +602,8 @@ impl Engine {
     ///
     /// The function signature must start with `&mut self` and not `&self`.
     ///
+    /// Not available under both `no_index` and `no_object`.
+    ///
     /// # Panics
     ///
     /// Panics if the type is [`Array`], [`Map`], [`String`], [`ImmutableString`][crate::ImmutableString] or `&str`.
@@ -621,11 +640,12 @@ impl Engine {
     ///     // Register an indexer.
     ///     .register_indexer_get_result(TestStruct::get_field);
     ///
+    /// # #[cfg(not(feature = "no_index"))]
     /// assert_eq!(engine.eval::<i64>("let a = new_ts(); a[2]")?, 3);
     /// # Ok(())
     /// # }
     /// ```
-    #[cfg(not(feature = "no_index"))]
+    #[cfg(any(not(feature = "no_index"), not(feature = "no_object")))]
     #[inline(always)]
     pub fn register_indexer_get_result<
         T: Variant + Clone,
@@ -635,6 +655,7 @@ impl Engine {
         &mut self,
         get_fn: impl Fn(&mut T, X) -> Result<V, Box<EvalAltResult>> + SendSync + 'static,
     ) -> &mut Self {
+        #[cfg(not(feature = "no_index"))]
         if TypeId::of::<T>() == TypeId::of::<Array>() {
             panic!("Cannot register indexer for arrays.");
         }
@@ -653,6 +674,8 @@ impl Engine {
     }
     /// Register an index setter for a custom type with the [`Engine`].
     ///
+    /// Not available under both `no_index` and `no_object`.
+    ///
     /// # Panics
     ///
     /// Panics if the type is [`Array`], [`Map`], [`String`], [`ImmutableString`][crate::ImmutableString] or `&str`.
@@ -686,6 +709,7 @@ impl Engine {
     ///     // Register an indexer.
     ///     .register_indexer_set(TestStruct::set_field);
     ///
+    /// # #[cfg(not(feature = "no_index"))]
     /// assert_eq!(
     ///     engine.eval::<TestStruct>("let a = new_ts(); a[2] = 42; a")?.fields[2],
     ///     42
@@ -693,12 +717,13 @@ impl Engine {
     /// # Ok(())
     /// # }
     /// ```
-    #[cfg(not(feature = "no_index"))]
+    #[cfg(any(not(feature = "no_index"), not(feature = "no_object")))]
     #[inline(always)]
     pub fn register_indexer_set<T: Variant + Clone, X: Variant + Clone, V: Variant + Clone>(
         &mut self,
         set_fn: impl Fn(&mut T, X, V) + SendSync + 'static,
     ) -> &mut Self {
+        #[cfg(not(feature = "no_index"))]
         if TypeId::of::<T>() == TypeId::of::<Array>() {
             panic!("Cannot register indexer for arrays.");
         }
@@ -717,6 +742,8 @@ impl Engine {
     }
     /// Register an index setter for a custom type with the [`Engine`].
     ///
+    /// Not available under both `no_index` and `no_object`.
+    ///
     /// # Panics
     ///
     /// Panics if the type is [`Array`], [`Map`], [`String`], [`ImmutableString`][crate::ImmutableString] or `&str`.
@@ -753,6 +780,7 @@ impl Engine {
     ///     // Register an indexer.
     ///     .register_indexer_set_result(TestStruct::set_field);
     ///
+    /// # #[cfg(not(feature = "no_index"))]
     /// assert_eq!(
     ///     engine.eval::<TestStruct>("let a = new_ts(); a[2] = 42; a")?.fields[2],
     ///     42
@@ -760,7 +788,7 @@ impl Engine {
     /// # Ok(())
     /// # }
     /// ```
-    #[cfg(not(feature = "no_index"))]
+    #[cfg(any(not(feature = "no_index"), not(feature = "no_object")))]
     #[inline(always)]
     pub fn register_indexer_set_result<
         T: Variant + Clone,
@@ -770,6 +798,7 @@ impl Engine {
         &mut self,
         set_fn: impl Fn(&mut T, X, V) -> Result<(), Box<EvalAltResult>> + SendSync + 'static,
     ) -> &mut Self {
+        #[cfg(not(feature = "no_index"))]
         if TypeId::of::<T>() == TypeId::of::<Array>() {
             panic!("Cannot register indexer for arrays.");
         }
@@ -784,12 +813,11 @@ impl Engine {
             panic!("Cannot register indexer for strings.");
         }
 
-        self.register_result_fn(
-            crate::engine::FN_IDX_SET,
-            move |obj: &mut T, index: X, value: V| set_fn(obj, index, value),
-        )
+        self.register_result_fn(crate::engine::FN_IDX_SET, set_fn)
     }
-    /// Short-hand for register both index getter and setter functions for a custom type with the [`Engine`].
+    /// Short-hand for registering both index getter and setter functions for a custom type with the [`Engine`].
+    ///
+    /// Not available under both `no_index` and `no_object`.
     ///
     /// # Panics
     ///
@@ -827,11 +855,12 @@ impl Engine {
     ///     // Register an indexer.
     ///     .register_indexer_get_set(TestStruct::get_field, TestStruct::set_field);
     ///
+    /// # #[cfg(not(feature = "no_index"))]
     /// assert_eq!(engine.eval::<i64>("let a = new_ts(); a[2] = 42; a[2]")?, 42);
     /// # Ok(())
     /// # }
     /// ```
-    #[cfg(not(feature = "no_index"))]
+    #[cfg(any(not(feature = "no_index"), not(feature = "no_object")))]
     #[inline(always)]
     pub fn register_indexer_get_set<T: Variant + Clone, X: Variant + Clone, V: Variant + Clone>(
         &mut self,
@@ -869,6 +898,8 @@ impl Engine {
     /// Functions marked [`FnNamespace::Global`] and type iterators are exposed to scripts without
     /// namespace qualifications.
     ///
+    /// Not available under `no_module`.
+    ///
     /// # Example
     ///
     /// ```
@@ -920,8 +951,14 @@ impl Engine {
                 }
             } else {
                 let mut iter = name.as_ref().splitn(2, separator.as_ref());
-                let sub_module = iter.next().unwrap().trim();
-                let remainder = iter.next().unwrap().trim();
+                let sub_module = iter
+                    .next()
+                    .expect("never fails because the name contains a separator")
+                    .trim();
+                let remainder = iter
+                    .next()
+                    .expect("never fails because the name contains a separator")
+                    .trim();
 
                 if !root.contains_key(sub_module) {
                     let mut m: Module = Default::default();
@@ -929,7 +966,9 @@ impl Engine {
                     m.build_index();
                     root.insert(sub_module.into(), m.into());
                 } else {
-                    let m = root.remove(sub_module).unwrap();
+                    let m = root
+                        .remove(sub_module)
+                        .expect("never fails because the root contains the sub-module");
                     let mut m = crate::fn_native::shared_take_or_clone(m);
                     register_static_module_raw(m.sub_modules_mut(), remainder, module);
                     m.build_index();
@@ -1023,6 +1062,8 @@ impl Engine {
     /// Compile a string into an [`AST`] using own scope, which can be used later for evaluation,
     /// embedding all imported modules.
     ///
+    /// Not available under `no_module`.
+    ///
     /// Modules referred by `import` statements containing literal string paths are eagerly resolved
     /// via the current [module resolver][crate::ModuleResolver] and embedded into the resultant
     /// [`AST`]. When it is evaluated later, `import` statement directly recall pre-resolved
@@ -1045,7 +1086,10 @@ impl Engine {
             resolver: &StaticModuleResolver,
             imports: &mut BTreeSet<Identifier>,
         ) {
-            ast.walk(&mut |path| match path.last().unwrap() {
+            ast.walk(&mut |path| match path
+                .last()
+                .expect("never fails because `path` always contains the current node")
+            {
                 // Collect all `import` statements with a string constant path
                 ASTNode::Stmt(Stmt::Import(Expr::StringConstant(s, _), _, _))
                     if !resolver.contains_path(s) && !imports.contains(s.as_str()) =>
@@ -1271,6 +1315,8 @@ impl Engine {
     /// Parse a JSON string into an [object map][`Map`].
     /// This is a light-weight alternative to using, say, [`serde_json`] to deserialize the JSON.
     ///
+    /// Not available under `no_object`.
+    ///
     /// The JSON string must be an object hash.  It cannot be a simple scalar value.
     ///
     /// Set `has_null` to `true` in order to map `null` values to `()`.
@@ -1787,6 +1833,8 @@ impl Engine {
     /// Call a script function defined in an [`AST`] with multiple arguments.
     /// Arguments are passed as a tuple.
     ///
+    /// Not available under `no_function`.
+    ///
     /// The [`AST`] is evaluated before calling the function.
     /// This allows a script to load the necessary modules.
     /// This is usually desired. If not, a specialized [`AST`] can be prepared that contains only
@@ -1854,6 +1902,8 @@ impl Engine {
     /// Call a script function defined in an [`AST`] with multiple [`Dynamic`] arguments
     /// and optionally a value for binding to the `this` pointer.
     ///
+    /// Not available under `no_function`.
+    ///
     /// There is an option to evaluate the [`AST`] to load necessary modules before calling the function.
     ///
     /// # WARNING
@@ -1970,6 +2020,8 @@ impl Engine {
     /// Optimize the [`AST`] with constants defined in an external Scope.
     /// An optimized copy of the [`AST`] is returned while the original [`AST`] is consumed.
     ///
+    /// Not available under `no_optimize`.
+    ///
     /// Although optimization is performed by default during compilation, sometimes it is necessary to
     /// _re_-optimize an [`AST`]. For example, when working with constants that are passed in via an
     /// external scope, it will be more efficient to optimize the [`AST`] once again to take advantage
@@ -2002,7 +2054,7 @@ impl Engine {
         let stmt = std::mem::take(ast.statements_mut());
         crate::optimize::optimize_into_ast(self, scope, stmt.into_vec(), lib, optimization_level)
     }
-    /// Generate a list of all registered functions.
+    /// _(METADATA)_ Generate a list of all registered functions.
     /// Exported under the `metadata` feature only.
     ///
     /// Functions from the following sources are included, in order:
@@ -2073,6 +2125,8 @@ impl Engine {
     }
     /// Register a callback for script evaluation progress.
     ///
+    /// Not available under `unchecked`.
+    ///
     /// # Example
     ///
     /// ```
