@@ -24,7 +24,7 @@ use std::prelude::v1::*;
 use std::{
     any::{type_name, TypeId},
     convert::TryFrom,
-    iter::{empty, once},
+    iter::once,
     mem,
 };
 
@@ -49,7 +49,12 @@ impl<'a> ArgBackup<'a> {
     ///
     /// This method blindly casts a reference to another lifetime, which saves allocation and string cloning.
     ///
-    /// If `restore_first_arg` is called before the end of the scope, the shorter lifetime will not leak.
+    /// As long as `restore_first_arg` is called before the end of the scope, the shorter lifetime
+    /// will not leak.
+    ///
+    /// # Panics
+    ///
+    /// Panics when `args` is empty.
     #[inline(always)]
     fn change_first_arg_to_copy(&mut self, args: &mut FnCallArgs<'a>) {
         // Clone the original value.
@@ -65,7 +70,7 @@ impl<'a> ArgBackup<'a> {
         //
         // We can do this here because, before the end of this scope, we'd restore the original reference
         // via `restore_first_arg`. Therefore this shorter lifetime does not leak.
-        self.orig_mut = Some(mem::replace(args.get_mut(0).unwrap(), unsafe {
+        self.orig_mut = Some(mem::replace(&mut args[0], unsafe {
             mem::transmute(&mut self.value_copy)
         }));
     }
@@ -77,9 +82,7 @@ impl<'a> ArgBackup<'a> {
     /// the current scope.  Otherwise it is undefined behavior as the shorter lifetime will leak.
     #[inline(always)]
     fn restore_first_arg(mut self, args: &mut FnCallArgs<'a>) {
-        if let Some(this_pointer) = self.orig_mut.take() {
-            args[0] = this_pointer;
-        }
+        self.orig_mut.take().map(|p| args[0] = p);
     }
 }
 
@@ -161,13 +164,11 @@ impl Engine {
         args: Option<&mut FnCallArgs>,
         allow_dynamic: bool,
         is_op_assignment: bool,
-    ) -> &'s Option<FnResolutionCacheEntry> {
-        let mut hash = if let Some(ref args) = args {
+    ) -> &'s Option<Box<FnResolutionCacheEntry>> {
+        let mut hash = args.as_ref().map_or(hash_script, |args| {
             let hash_params = calc_fn_params_hash(args.iter().map(|a| a.type_id()));
             combine_hashes(hash_script, hash_params)
-        } else {
-            hash_script
-        };
+        });
 
         &*state
             .fn_resolution_cache_mut()
@@ -222,7 +223,7 @@ impl Engine {
 
                     match func {
                         // Specific version found
-                        Some(f) => return Some(f),
+                        Some(f) => return Some(Box::new(f)),
 
                         // Stop when all permutations are exhausted
                         None if bitmask >= max_bitmask => {
@@ -239,7 +240,8 @@ impl Engine {
                                         FnResolutionCacheEntry { func, source: None }
                                     })
                                 } else {
-                                    let (first, second) = args.split_first().unwrap();
+                                    let (first, second) = args.split_first()
+                                                              .expect("never fails because an op-assignment must have two arguments");
 
                                     get_builtin_op_assignment_fn(fn_name, *first, second[0]).map(
                                         |f| {
@@ -250,13 +252,16 @@ impl Engine {
                                         },
                                     )
                                 }
+                                .map(Box::new)
                             });
                         }
 
                         // Try all permutations with `Dynamic` wildcards
                         None => {
                             let hash_params = calc_fn_params_hash(
-                                args.as_ref().unwrap().iter().enumerate().map(|(i, a)| {
+                                args.as_ref().expect("never fails because there are no permutations if there are no arguments")
+                                        .iter().enumerate().map(|(i, a)|
+                                {
                                     let mask = 1usize << (num_args - i - 1);
                                     if bitmask & mask != 0 {
                                         // Replace with `Dynamic`
@@ -311,14 +316,15 @@ impl Engine {
             is_op_assignment,
         );
 
-        if let Some(FnResolutionCacheEntry { func, source }) = func {
+        if let Some(f) = func {
+            let FnResolutionCacheEntry { func, source } = f.as_ref();
             assert!(func.is_native());
 
             // Calling pure function but the first argument is a reference?
             let mut backup: Option<ArgBackup> = None;
             if is_ref && func.is_pure() && !args.is_empty() {
                 backup = Some(Default::default());
-                backup.as_mut().unwrap().change_first_arg_to_copy(args);
+                backup.as_mut().map(|bk| bk.change_first_arg_to_copy(args));
             }
 
             // Run external function
@@ -334,9 +340,7 @@ impl Engine {
             };
 
             // Restore the original reference
-            if let Some(backup) = backup {
-                backup.restore_first_arg(args);
-            }
+            backup.map(|bk| bk.restore_first_arg(args));
 
             let result = result.map_err(|err| err.fill_position(pos))?;
 
@@ -369,32 +373,24 @@ impl Engine {
 
         match fn_name {
             // index getter function not found?
-            #[cfg(not(feature = "no_index"))]
+            #[cfg(any(not(feature = "no_index"), not(feature = "no_object")))]
             crate::engine::FN_IDX_GET => {
                 assert!(args.len() == 2);
 
-                EvalAltResult::ErrorFunctionNotFound(
-                    format!(
-                        "{} [{}]",
-                        self.map_type_name(args[0].type_name()),
-                        self.map_type_name(args[1].type_name()),
-                    ),
+                EvalAltResult::ErrorIndexingType(
+                    self.map_type_name(args[0].type_name()).to_string(),
                     pos,
                 )
                 .into()
             }
 
             // index setter function not found?
-            #[cfg(not(feature = "no_index"))]
+            #[cfg(any(not(feature = "no_index"), not(feature = "no_object")))]
             crate::engine::FN_IDX_SET => {
                 assert!(args.len() == 3);
 
-                EvalAltResult::ErrorFunctionNotFound(
-                    format!(
-                        "{} [{}]=",
-                        self.map_type_name(args[0].type_name()),
-                        self.map_type_name(args[1].type_name()),
-                    ),
+                EvalAltResult::ErrorIndexingType(
+                    self.map_type_name(args[0].type_name()).to_string(),
                     pos,
                 )
                 .into()
@@ -652,14 +648,18 @@ impl Engine {
             crate::engine::KEYWORD_IS_DEF_FN
                 if args.len() == 2 && args[0].is::<FnPtr>() && args[1].is::<crate::INT>() =>
             {
-                let fn_name = &*args[0].read_lock::<ImmutableString>().unwrap();
-                let num_params = args[1].as_int().unwrap();
+                let fn_name = &*args[0]
+                    .read_lock::<ImmutableString>()
+                    .expect("never fails because `args[0]` is `FnPtr`");
+                let num_params = args[1]
+                    .as_int()
+                    .expect("never fails because `args[1]` is `INT`");
 
                 return Ok((
                     if num_params < 0 {
                         Dynamic::FALSE
                     } else {
-                        let hash_script = calc_fn_hash(empty(), fn_name, num_params as usize);
+                        let hash_script = calc_fn_hash(fn_name, num_params as usize);
                         self.has_script_fn(Some(mods), state, lib, hash_script)
                             .into()
                     },
@@ -710,17 +710,14 @@ impl Engine {
 
         // Scripted function call?
         #[cfg(not(feature = "no_function"))]
-        let hash_script = if hash.is_native_only() {
-            None
-        } else {
-            Some(hash.script_hash())
-        };
+        let hash_script = hash.script_hash();
 
         #[cfg(not(feature = "no_function"))]
-        if let Some(FnResolutionCacheEntry { func, source }) = hash_script.and_then(|hash| {
+        if let Some(f) = hash_script.and_then(|hash| {
             self.resolve_function(mods, state, lib, fn_name, hash, None, false, false)
                 .clone()
         }) {
+            let FnResolutionCacheEntry { func, source } = *f;
             // Script function call
             assert!(func.is_script());
 
@@ -734,21 +731,23 @@ impl Engine {
 
             // Move captured variables into scope
             #[cfg(not(feature = "no_closure"))]
-            if let Some(captured) = _capture_scope {
-                if !func.externals.is_empty() {
+            if !func.externals.is_empty() {
+                _capture_scope.map(|captured| {
                     captured
                         .into_iter()
                         .filter(|(name, _, _)| func.externals.contains(name.as_ref()))
                         .for_each(|(name, value, _)| {
                             // Consume the scope values.
                             scope.push_dynamic(name, value);
-                        });
-                }
+                        })
+                });
             }
 
             let result = if _is_method {
                 // Method call of script function - map first argument to `this`
-                let (first, rest) = args.split_first_mut().unwrap();
+                let (first, rest) = args
+                    .split_first_mut()
+                    .expect("never fails because a method call always has a first parameter");
 
                 let orig_source = state.source.take();
                 state.source = source;
@@ -777,7 +776,7 @@ impl Engine {
                 let mut backup: Option<ArgBackup> = None;
                 if is_ref && !args.is_empty() {
                     backup = Some(Default::default());
-                    backup.as_mut().unwrap().change_first_arg_to_copy(args);
+                    backup.as_mut().map(|bk| bk.change_first_arg_to_copy(args));
                 }
 
                 let orig_source = state.source.take();
@@ -792,9 +791,7 @@ impl Engine {
                 state.source = orig_source;
 
                 // Restore the original reference
-                if let Some(backup) = backup {
-                    backup.restore_first_arg(args);
-                }
+                backup.map(|bk| bk.restore_first_arg(args));
 
                 result?
             };
@@ -911,12 +908,14 @@ impl Engine {
         let (result, updated) = match fn_name {
             KEYWORD_FN_PTR_CALL if obj.is::<FnPtr>() => {
                 // FnPtr call
-                let fn_ptr = obj.read_lock::<FnPtr>().unwrap();
+                let fn_ptr = obj
+                    .read_lock::<FnPtr>()
+                    .expect("never fails because `obj` is `FnPtr`");
                 // Redirect function name
                 let fn_name = fn_ptr.fn_name();
                 let args_len = call_args.len() + fn_ptr.curry().len();
                 // Recalculate hashes
-                let new_hash = FnCallHashes::from_script(calc_fn_hash(empty(), fn_name, args_len));
+                let new_hash = FnCallHashes::from_script(calc_fn_hash(fn_name, args_len));
                 // Arguments are passed as-is, adding the curried arguments
                 let mut curry = fn_ptr.curry().iter().cloned().collect::<StaticVec<_>>();
                 let mut args = curry
@@ -952,8 +951,8 @@ impl Engine {
                 let args_len = call_args.len() + fn_ptr.curry().len();
                 // Recalculate hash
                 let new_hash = FnCallHashes::from_script_and_native(
-                    calc_fn_hash(empty(), fn_name, args_len),
-                    calc_fn_hash(empty(), fn_name, args_len + 1),
+                    calc_fn_hash(fn_name, args_len),
+                    calc_fn_hash(fn_name, args_len + 1),
                 );
                 // Replace the first argument with the object pointer, adding the curried arguments
                 let mut curry = fn_ptr.curry().iter().cloned().collect::<StaticVec<_>>();
@@ -975,7 +974,9 @@ impl Engine {
                     ));
                 }
 
-                let fn_ptr = obj.read_lock::<FnPtr>().unwrap();
+                let fn_ptr = obj
+                    .read_lock::<FnPtr>()
+                    .expect("never fails because `obj` is `FnPtr`");
 
                 // Curry call
                 Ok((
@@ -1026,8 +1027,8 @@ impl Engine {
                                 });
                             // Recalculate the hash based on the new function name and new arguments
                             hash = FnCallHashes::from_script_and_native(
-                                calc_fn_hash(empty(), fn_name, call_args.len()),
-                                calc_fn_hash(empty(), fn_name, call_args.len() + 1),
+                                calc_fn_hash(fn_name, call_args.len()),
+                                calc_fn_hash(fn_name, call_args.len() + 1),
                             );
                         }
                     }
@@ -1046,7 +1047,9 @@ impl Engine {
 
         // Propagate the changed value back to the source if necessary
         if updated {
-            target.propagate_changed_value();
+            target
+                .propagate_changed_value()
+                .map_err(|err| err.fill_position(pos))?;
         }
 
         Ok((result, updated))
@@ -1062,7 +1065,7 @@ impl Engine {
         this_ptr: &mut Option<&mut Dynamic>,
         fn_name: &str,
         args_expr: &[Expr],
-        constant_args: &[(Dynamic, Position)],
+        literal_args: &[(Dynamic, Position)],
         mut hashes: FnCallHashes,
         pos: Position,
         capture_scope: bool,
@@ -1071,8 +1074,8 @@ impl Engine {
         // Handle call() - Redirect function call
         let redirected;
         let mut args_expr = args_expr;
-        let mut constant_args = constant_args;
-        let mut total_args = args_expr.len() + constant_args.len();
+        let mut literal_args = literal_args;
+        let mut total_args = args_expr.len() + literal_args.len();
         let mut curry = StaticVec::new();
         let mut name = fn_name;
 
@@ -1080,7 +1083,7 @@ impl Engine {
             // Handle call()
             KEYWORD_FN_PTR_CALL if total_args >= 1 => {
                 let (arg, arg_pos) = args_expr.get(0).map_or_else(
-                    || Ok(constant_args[0].clone()),
+                    || Ok(literal_args[0].clone()),
                     |arg| {
                         self.eval_expr(scope, mods, state, lib, this_ptr, arg, level)
                             .map(|v| (v, arg.position()))
@@ -1105,22 +1108,22 @@ impl Engine {
                 if !args_expr.is_empty() {
                     args_expr = &args_expr[1..];
                 } else {
-                    constant_args = &constant_args[1..];
+                    literal_args = &literal_args[1..];
                 }
                 total_args -= 1;
 
                 // Recalculate hash
                 let args_len = total_args + curry.len();
                 hashes = if !hashes.is_native_only() {
-                    FnCallHashes::from_script(calc_fn_hash(empty(), name, args_len))
+                    FnCallHashes::from_script(calc_fn_hash(name, args_len))
                 } else {
-                    FnCallHashes::from_native(calc_fn_hash(empty(), name, args_len))
+                    FnCallHashes::from_native(calc_fn_hash(name, args_len))
                 };
             }
             // Handle Fn()
             KEYWORD_FN_PTR if total_args == 1 => {
                 let (arg, arg_pos) = args_expr.get(0).map_or_else(
-                    || Ok(constant_args[0].clone()),
+                    || Ok(literal_args[0].clone()),
                     |arg| {
                         self.eval_expr(scope, mods, state, lib, this_ptr, arg, level)
                             .map(|v| (v, arg.position()))
@@ -1139,7 +1142,7 @@ impl Engine {
             // Handle curry()
             KEYWORD_FN_PTR_CURRY if total_args > 1 => {
                 let (arg, arg_pos) = args_expr.get(0).map_or_else(
-                    || Ok(constant_args[0].clone()),
+                    || Ok(literal_args[0].clone()),
                     |arg| {
                         self.eval_expr(scope, mods, state, lib, this_ptr, arg, level)
                             .map(|v| (v, arg.position()))
@@ -1161,9 +1164,9 @@ impl Engine {
                         self.eval_expr(scope, mods, state, lib, this_ptr, expr, level)
                             .map(|value| fn_curry.push(value))
                     })?;
-                    fn_curry.extend(constant_args.iter().map(|(v, _)| v.clone()));
+                    fn_curry.extend(literal_args.iter().map(|(v, _)| v.clone()));
                 } else {
-                    fn_curry.extend(constant_args.iter().skip(1).map(|(v, _)| v.clone()));
+                    fn_curry.extend(literal_args.iter().skip(1).map(|(v, _)| v.clone()));
                 }
 
                 return Ok(FnPtr::new_unchecked(name, fn_curry).into());
@@ -1173,7 +1176,7 @@ impl Engine {
             #[cfg(not(feature = "no_closure"))]
             crate::engine::KEYWORD_IS_SHARED if total_args == 1 => {
                 let arg = args_expr.get(0).map_or_else(
-                    || Ok(constant_args[0].0.clone()),
+                    || Ok(literal_args[0].0.clone()),
                     |arg| self.eval_expr(scope, mods, state, lib, this_ptr, arg, level),
                 )?;
                 return Ok(arg.is_shared().into());
@@ -1188,7 +1191,7 @@ impl Engine {
                         args_expr[0].position(),
                     )
                 } else {
-                    constant_args[0].clone()
+                    literal_args[0].clone()
                 };
 
                 let fn_name = arg
@@ -1201,7 +1204,7 @@ impl Engine {
                         args_expr[1].position(),
                     )
                 } else {
-                    constant_args[if args_expr.is_empty() { 1 } else { 0 }].clone()
+                    literal_args[if args_expr.is_empty() { 1 } else { 0 }].clone()
                 };
 
                 let num_params = arg
@@ -1211,7 +1214,7 @@ impl Engine {
                 return Ok(if num_params < 0 {
                     Dynamic::FALSE
                 } else {
-                    let hash_script = calc_fn_hash(empty(), &fn_name, num_params as usize);
+                    let hash_script = calc_fn_hash(&fn_name, num_params as usize);
                     self.has_script_fn(Some(mods), state, lib, hash_script)
                         .into()
                 });
@@ -1220,7 +1223,7 @@ impl Engine {
             // Handle is_def_var()
             KEYWORD_IS_DEF_VAR if total_args == 1 => {
                 let (arg, arg_pos) = args_expr.get(0).map_or_else(
-                    || Ok(constant_args[0].clone()),
+                    || Ok(literal_args[0].clone()),
                     |arg| {
                         self.eval_expr(scope, mods, state, lib, this_ptr, arg, level)
                             .map(|v| (v, arg.position()))
@@ -1237,7 +1240,7 @@ impl Engine {
                 // eval - only in function call style
                 let prev_len = scope.len();
                 let (script, script_pos) = args_expr.get(0).map_or_else(
-                    || Ok(constant_args[0].clone()),
+                    || Ok(literal_args[0].clone()),
                     |script_expr| {
                         self.eval_expr(scope, mods, state, lib, this_ptr, script_expr, level)
                             .map(|v| (v, script_expr.position()))
@@ -1289,7 +1292,7 @@ impl Engine {
             None
         };
 
-        if args_expr.is_empty() && constant_args.is_empty() && curry.is_empty() {
+        if args_expr.is_empty() && literal_args.is_empty() && curry.is_empty() {
             // No arguments
             args = Default::default();
         } else {
@@ -1305,7 +1308,7 @@ impl Engine {
                         self.eval_expr(scope, mods, state, lib, this_ptr, expr, level)
                             .map(Dynamic::flatten)
                     })
-                    .chain(constant_args.iter().map(|(v, _)| Ok(v.clone())))
+                    .chain(literal_args.iter().map(|(v, _)| Ok(v.clone())))
                     .collect::<Result<_, _>>()?;
 
                 let (mut target, _pos) =
@@ -1329,9 +1332,8 @@ impl Engine {
                 } else {
                     // Turn it into a method call only if the object is not shared and not a simple value
                     is_ref = true;
-                    once(target.take_ref().unwrap())
-                        .chain(arg_values.iter_mut())
-                        .collect()
+                    let obj_ref = target.take_ref().expect("never fails because `target` is a reference if it is not a value and not shared");
+                    once(obj_ref).chain(arg_values.iter_mut()).collect()
                 };
             } else {
                 // func(..., ...)
@@ -1341,7 +1343,7 @@ impl Engine {
                         self.eval_expr(scope, mods, state, lib, this_ptr, expr, level)
                             .map(Dynamic::flatten)
                     })
-                    .chain(constant_args.iter().map(|(v, _)| Ok(v.clone())))
+                    .chain(literal_args.iter().map(|(v, _)| Ok(v.clone())))
                     .collect::<Result<_, _>>()?;
 
                 args = curry.iter_mut().chain(arg_values.iter_mut()).collect();
@@ -1362,20 +1364,19 @@ impl Engine {
         state: &mut State,
         lib: &[&Module],
         this_ptr: &mut Option<&mut Dynamic>,
-        namespace: Option<&NamespaceRef>,
+        namespace: &NamespaceRef,
         fn_name: &str,
         args_expr: &[Expr],
-        constant_args: &[(Dynamic, Position)],
+        literal_args: &[(Dynamic, Position)],
         hash: u64,
         pos: Position,
         level: usize,
     ) -> RhaiResult {
-        let namespace = namespace.unwrap();
         let mut arg_values: StaticVec<_>;
         let mut first_arg_value = None;
         let mut args: StaticVec<_>;
 
-        if args_expr.is_empty() && constant_args.is_empty() {
+        if args_expr.is_empty() && literal_args.is_empty() {
             // No arguments
             args = Default::default();
         } else {
@@ -1396,7 +1397,7 @@ impl Engine {
                                 .map(Dynamic::flatten)
                         }
                     })
-                    .chain(constant_args.iter().map(|(v, _)| Ok(v.clone())))
+                    .chain(literal_args.iter().map(|(v, _)| Ok(v.clone())))
                     .collect::<Result<_, _>>()?;
 
                 // Get target reference to first argument
@@ -1415,11 +1416,12 @@ impl Engine {
                     arg_values[0] = target.take_or_clone().flatten();
                     args = arg_values.iter_mut().collect();
                 } else {
-                    let (first, rest) = arg_values.split_first_mut().unwrap();
+                    let (first, rest) = arg_values
+                        .split_first_mut()
+                        .expect("never fails because the arguments list is not empty");
                     first_arg_value = Some(first);
-                    args = once(target.take_ref().unwrap())
-                        .chain(rest.iter_mut())
-                        .collect();
+                    let obj_ref = target.take_ref().expect("never fails because `target` is a reference if it is not a value and not shared");
+                    args = once(obj_ref).chain(rest.iter_mut()).collect();
                 }
             } else {
                 // func(..., ...) or func(mod::x, ...)
@@ -1429,7 +1431,7 @@ impl Engine {
                         self.eval_expr(scope, mods, state, lib, this_ptr, expr, level)
                             .map(Dynamic::flatten)
                     })
-                    .chain(constant_args.iter().map(|(v, _)| Ok(v.clone())))
+                    .chain(literal_args.iter().map(|(v, _)| Ok(v.clone())))
                     .collect::<Result<_, _>>()?;
 
                 args = arg_values.iter_mut().collect();
@@ -1456,12 +1458,11 @@ impl Engine {
         };
 
         // Clone first argument if the function is not a method after-all
-        if let Some(first) = first_arg_value {
-            if !func.map(|f| f.is_method()).unwrap_or(true) {
-                let first_val = args[0].clone();
+        if !func.map(|f| f.is_method()).unwrap_or(true) {
+            first_arg_value.map(|first| {
+                *first = args[0].clone();
                 args[0] = first;
-                *args[0] = first_val;
-            }
+            });
         }
 
         match func {
