@@ -5,6 +5,7 @@
 use crate::dynamic::{DynamicWriteLock, Variant};
 use crate::fn_native::{CallableFunction, FnAny, FnCallArgs, SendSync};
 use crate::r#unsafe::unsafe_try_cast;
+use crate::token::Position;
 use crate::{Dynamic, EvalAltResult, NativeCallContext};
 #[cfg(feature = "no_std")]
 use std::prelude::v1::*;
@@ -31,7 +32,8 @@ pub struct Mut<T>(T);
 #[inline(always)]
 pub fn by_ref<T: Variant + Clone>(data: &mut Dynamic) -> DynamicWriteLock<T> {
     // Directly cast the &mut Dynamic into DynamicWriteLock to access the underlying data.
-    data.write_lock::<T>().unwrap()
+    data.write_lock::<T>()
+        .expect("never fails because the type was checked")
 }
 
 /// Dereference into value.
@@ -42,12 +44,15 @@ pub fn by_value<T: Variant + Clone>(data: &mut Dynamic) -> T {
         data.flatten_in_place();
         let ref_str = data
             .as_str_ref()
-            .expect("argument passed by value should not be shared");
+            .expect("never fails because argument passed by value should not be shared");
         let ref_t = unsafe { mem::transmute::<_, &T>(&ref_str) };
         ref_t.clone()
     } else if TypeId::of::<T>() == TypeId::of::<String>() {
         // If T is `String`, data must be `ImmutableString`, so map directly to it
-        unsafe_try_cast(mem::take(data).take_string().unwrap()).unwrap()
+        let value = mem::take(data)
+            .take_string()
+            .expect("never fails because the type was checked");
+        unsafe_try_cast(value).expect("never fails because the type was checked")
     } else {
         // We consume the argument and then replace it with () - the argument is not supposed to be used again.
         // This way, we avoid having to clone the argument again, because it is already a clone when passed here.
@@ -61,20 +66,33 @@ pub trait RegisterNativeFunction<Args, Result> {
     fn into_callable_function(self) -> CallableFunction;
     /// Get the type ID's of this function's parameters.
     fn param_types() -> Box<[TypeId]>;
-    /// Get the type names of this function's parameters.
+    /// _(METADATA)_ Get the type names of this function's parameters.
     /// Exported under the `metadata` feature only.
     #[cfg(feature = "metadata")]
     fn param_names() -> Box<[&'static str]>;
-    /// Get the type ID of this function's return value.
+    /// _(METADATA)_ Get the type ID of this function's return value.
     /// Exported under the `metadata` feature only.
     #[cfg(feature = "metadata")]
     fn return_type() -> TypeId;
-    /// Get the type name of this function's return value.
+    /// _(METADATA)_ Get the type name of this function's return value.
     /// Exported under the `metadata` feature only.
     #[cfg(feature = "metadata")]
     fn return_type_name() -> &'static str;
 }
 
+#[inline(always)]
+fn is_setter(_fn_name: &str) -> bool {
+    #[cfg(not(feature = "no_object"))]
+    if _fn_name.starts_with(crate::engine::FN_SET) {
+        return true;
+    }
+    #[cfg(not(feature = "no_index"))]
+    if _fn_name.starts_with(crate::engine::FN_IDX_SET) {
+        return true;
+    }
+    false
+}
+
 macro_rules! def_register {
     () => {
         def_register!(imp from_pure :);
@@ -97,10 +115,14 @@ macro_rules! def_register {
             #[cfg(feature = "metadata")] #[inline(always)] fn return_type() -> TypeId { TypeId::of::<RET>() }
             #[cfg(feature = "metadata")] #[inline(always)] fn return_type_name() -> &'static str { std::any::type_name::<RET>() }
             #[inline(always)] fn into_callable_function(self) -> CallableFunction {
-                CallableFunction::$abi(Box::new(move |_: NativeCallContext, args: &mut FnCallArgs| {
+                CallableFunction::$abi(Box::new(move |ctx: NativeCallContext, args: &mut FnCallArgs| {
+                    if args.len() == 2 && args[0].is_read_only() && is_setter(ctx.fn_name()) {
+                        return EvalAltResult::ErrorAssignmentToConstant(Default::default(), Position::NONE).into();
+                    }
+
                     // The arguments are assumed to be of the correct number and types!
                     let mut _drain = args.iter_mut();
-                    $($let $par = ($clone)(_drain.next().unwrap()); )*
+                    $($let $par = ($clone)(_drain.next().expect("never fails because arguments list is fixed")); )*
 
                     // Call the function with each argument value
                     let r = self($($arg),*);
@@ -122,9 +144,13 @@ macro_rules! def_register {
             #[cfg(feature = "metadata")] #[inline(always)] fn return_type_name() -> &'static str { std::any::type_name::<RET>() }
             #[inline(always)] fn into_callable_function(self) -> CallableFunction {
                 CallableFunction::$abi(Box::new(move |ctx: NativeCallContext, args: &mut FnCallArgs| {
+                    if args.len() == 2 && args[0].is_read_only() && is_setter(ctx.fn_name()) {
+                        return EvalAltResult::ErrorAssignmentToConstant(Default::default(), Position::NONE).into();
+                    }
+
                     // The arguments are assumed to be of the correct number and types!
                     let mut _drain = args.iter_mut();
-                    $($let $par = ($clone)(_drain.next().unwrap()); )*
+                    $($let $par = ($clone)(_drain.next().expect("never fails because arguments list is fixed")); )*
 
                     // Call the function with each argument value
                     let r = self(ctx, $($arg),*);
@@ -145,10 +171,14 @@ macro_rules! def_register {
             #[cfg(feature = "metadata")] #[inline(always)] fn return_type() -> TypeId { TypeId::of::<Result<RET, Box<EvalAltResult>>>() }
             #[cfg(feature = "metadata")] #[inline(always)] fn return_type_name() -> &'static str { std::any::type_name::<Result<RET, Box<EvalAltResult>>>() }
             #[inline(always)] fn into_callable_function(self) -> CallableFunction {
-                CallableFunction::$abi(Box::new(move |_: NativeCallContext, args: &mut FnCallArgs| {
+                CallableFunction::$abi(Box::new(move |ctx: NativeCallContext, args: &mut FnCallArgs| {
+                    if args.len() == 2 && args[0].is_read_only() && is_setter(ctx.fn_name()) {
+                        return EvalAltResult::ErrorAssignmentToConstant(Default::default(), Position::NONE).into();
+                    }
+
                     // The arguments are assumed to be of the correct number and types!
                     let mut _drain = args.iter_mut();
-                    $($let $par = ($clone)(_drain.next().unwrap()); )*
+                    $($let $par = ($clone)(_drain.next().expect("never fails because arguments list is fixed")); )*
 
                     // Call the function with each argument value
                     self($($arg),*).map(Dynamic::from)
@@ -167,9 +197,13 @@ macro_rules! def_register {
             #[cfg(feature = "metadata")] #[inline(always)] fn return_type_name() -> &'static str { std::any::type_name::<Result<RET, Box<EvalAltResult>>>() }
             #[inline(always)] fn into_callable_function(self) -> CallableFunction {
                 CallableFunction::$abi(Box::new(move |ctx: NativeCallContext, args: &mut FnCallArgs| {
+                    if args.len() == 2 && args[0].is_read_only() && is_setter(ctx.fn_name()) {
+                        return EvalAltResult::ErrorAssignmentToConstant(Default::default(), Position::NONE).into();
+                    }
+
                     // The arguments are assumed to be of the correct number and types!
                     let mut _drain = args.iter_mut();
-                    $($let $par = ($clone)(_drain.next().unwrap()); )*
+                    $($let $par = ($clone)(_drain.next().expect("never fails because arguments list is fixed")); )*
 
                     // Call the function with each argument value
                     self(ctx, $($arg),*).map(Dynamic::from)
