@@ -7,8 +7,8 @@ use crate::fn_register::RegisterNativeFunction;
 use crate::token::Token;
 use crate::utils::IdentifierBuilder;
 use crate::{
-    calc_fn_hash, calc_fn_params_hash, combine_hashes, Dynamic, EvalAltResult, Identifier,
-    ImmutableString, NativeCallContext, Position, Shared, StaticVec,
+    calc_fn_params_hash, calc_qualified_fn_hash, combine_hashes, Dynamic, EvalAltResult,
+    Identifier, ImmutableString, NativeCallContext, Position, Shared, StaticVec,
 };
 #[cfg(feature = "no_std")]
 use std::prelude::v1::*;
@@ -24,7 +24,6 @@ use std::{
 #[cfg(not(feature = "no_index"))]
 use crate::Array;
 
-#[cfg(not(feature = "no_index"))]
 #[cfg(not(feature = "no_object"))]
 use crate::Map;
 
@@ -48,7 +47,7 @@ impl Default for FnNamespace {
 #[derive(Debug, Clone)]
 pub struct FuncInfo {
     /// Function instance.
-    pub func: CallableFunction,
+    pub func: Shared<CallableFunction>,
     /// Function namespace.
     pub namespace: FnNamespace,
     /// Function access mode.
@@ -117,7 +116,7 @@ fn calc_native_fn_hash<'a>(
     fn_name: impl AsRef<str>,
     params: &[TypeId],
 ) -> u64 {
-    let hash_script = calc_fn_hash(modules, fn_name, params.len());
+    let hash_script = calc_qualified_fn_hash(modules, fn_name, params.len());
     let hash_params = calc_fn_params_hash(params.iter().cloned());
     combine_hashes(hash_script, hash_params)
 }
@@ -129,7 +128,7 @@ pub struct Module {
     /// ID identifying the module.
     id: Option<Identifier>,
     /// Is this module internal?
-    internal: bool,
+    pub(crate) internal: bool,
     /// Sub-modules.
     modules: BTreeMap<Identifier, Shared<Module>>,
     /// [`Module`] variables.
@@ -140,7 +139,7 @@ pub struct Module {
     functions: BTreeMap<u64, Box<FuncInfo>>,
     /// Flattened collection of all external Rust functions, native or scripted.
     /// including those in sub-modules.
-    all_functions: BTreeMap<u64, CallableFunction>,
+    all_functions: BTreeMap<u64, Shared<CallableFunction>>,
     /// Iterator functions, keyed by the type producing the iterator.
     type_iterators: BTreeMap<TypeId, IteratorFn>,
     /// Flattened collection of iterator functions, including those in sub-modules.
@@ -156,20 +155,7 @@ pub struct Module {
 impl Default for Module {
     #[inline(always)]
     fn default() -> Self {
-        Self {
-            id: None,
-            internal: false,
-            modules: Default::default(),
-            variables: Default::default(),
-            all_variables: Default::default(),
-            functions: Default::default(),
-            all_functions: Default::default(),
-            type_iterators: Default::default(),
-            all_type_iterators: Default::default(),
-            indexed: true,
-            contains_indexed_global_functions: false,
-            identifiers: Default::default(),
-        }
+        Self::new()
     }
 }
 
@@ -177,9 +163,7 @@ impl fmt::Debug for Module {
     fn fmt(&self, f: &mut fmt::Formatter<'_>) -> fmt::Result {
         let mut d = f.debug_struct("Module");
 
-        if let Some(ref id) = self.id {
-            d.field("id", id);
-        }
+        self.id.as_ref().map(|id| d.field("id", id));
 
         if !self.modules.is_empty() {
             d.field(
@@ -257,7 +241,20 @@ impl Module {
     /// ```
     #[inline(always)]
     pub fn new() -> Self {
-        Default::default()
+        Self {
+            id: None,
+            internal: false,
+            modules: Default::default(),
+            variables: Default::default(),
+            all_variables: Default::default(),
+            functions: Default::default(),
+            all_functions: Default::default(),
+            type_iterators: Default::default(),
+            all_type_iterators: Default::default(),
+            indexed: true,
+            contains_indexed_global_functions: false,
+            identifiers: Default::default(),
+        }
     }
 
     /// Get the ID of the [`Module`], if any.
@@ -309,20 +306,6 @@ impl Module {
         self
     }
 
-    /// Is the [`Module`] internal?
-    #[allow(dead_code)]
-    #[inline(always)]
-    pub(crate) fn is_internal(&self) -> bool {
-        self.internal
-    }
-
-    /// Set the internal status of the [`Module`].
-    #[inline(always)]
-    pub(crate) fn set_internal(&mut self, value: bool) -> &mut Self {
-        self.internal = value;
-        self
-    }
-
     /// Is the [`Module`] empty?
     ///
     /// # Example
@@ -455,7 +438,7 @@ impl Module {
         let value = Dynamic::from(value);
 
         if self.indexed {
-            let hash_var = crate::calc_fn_hash(once(""), &ident, 0);
+            let hash_var = crate::calc_qualified_fn_hash(once(""), &ident, 0);
             self.all_variables.insert(hash_var, value.clone());
         }
         self.variables.insert(ident, value);
@@ -476,15 +459,12 @@ impl Module {
     /// If there is an existing function of the same name and number of arguments, it is replaced.
     #[cfg(not(feature = "no_function"))]
     #[inline]
-    pub(crate) fn set_script_fn(
-        &mut self,
-        fn_def: impl Into<Shared<crate::ast::ScriptFnDef>>,
-    ) -> u64 {
+    pub fn set_script_fn(&mut self, fn_def: impl Into<Shared<crate::ast::ScriptFnDef>>) -> u64 {
         let fn_def = fn_def.into();
 
         // None + function name + number of arguments.
         let num_params = fn_def.params.len();
-        let hash_script = crate::calc_fn_hash(empty(), &fn_def.name, num_params);
+        let hash_script = crate::calc_fn_hash(&fn_def.name, num_params);
         let mut param_names = fn_def.params.clone();
         param_names.push("Dynamic".into());
         self.functions.insert(
@@ -497,7 +477,7 @@ impl Module {
                 param_types: Default::default(),
                 #[cfg(feature = "metadata")]
                 param_names,
-                func: fn_def.into(),
+                func: Into::<CallableFunction>::into(fn_def).into(),
             }),
         );
         self.indexed = false;
@@ -639,9 +619,10 @@ impl Module {
             .map(|&name| self.identifiers.get(name))
             .collect();
 
-        if let Some(f) = self.functions.get_mut(&hash_fn) {
-            f.param_names = param_names;
-        }
+        self.functions
+            .get_mut(&hash_fn)
+            .map(|f| f.param_names = param_names);
+
         self
     }
 
@@ -695,19 +676,22 @@ impl Module {
     ) -> u64 {
         let is_method = func.is_method();
 
-        let param_types: StaticVec<_> = arg_types
+        let mut param_types: StaticVec<_> = arg_types
             .iter()
             .cloned()
             .enumerate()
             .map(|(i, type_id)| Self::map_type(!is_method || i > 0, type_id))
             .collect();
+        param_types.shrink_to_fit();
 
         #[cfg(feature = "metadata")]
-        let param_names = _arg_names
+        let mut param_names: StaticVec<_> = _arg_names
             .iter()
             .flat_map(|p| p.iter())
             .map(|&arg| self.identifiers.get(arg))
             .collect();
+        #[cfg(feature = "metadata")]
+        param_names.shrink_to_fit();
 
         let hash_fn = calc_native_fn_hash(empty(), &name, &param_types);
 
@@ -721,7 +705,7 @@ impl Module {
                 param_types,
                 #[cfg(feature = "metadata")]
                 param_names,
-                func,
+                func: func.into(),
             }),
         );
 
@@ -970,7 +954,7 @@ impl Module {
     /// });
     /// assert!(module.contains_fn(hash));
     /// ```
-    #[cfg(not(feature = "no_index"))]
+    #[cfg(any(not(feature = "no_index"), not(feature = "no_object")))]
     #[inline(always)]
     pub fn set_indexer_get_fn<ARGS, A, B, T, F>(&mut self, func: F) -> u64
     where
@@ -980,6 +964,7 @@ impl Module {
         F: RegisterNativeFunction<ARGS, Result<T, Box<EvalAltResult>>>,
         F: Fn(&mut A, B) -> Result<T, Box<EvalAltResult>> + SendSync + 'static,
     {
+        #[cfg(not(feature = "no_index"))]
         if TypeId::of::<A>() == TypeId::of::<Array>() {
             panic!("Cannot register indexer for arrays.");
         }
@@ -1030,7 +1015,7 @@ impl Module {
     /// });
     /// assert!(module.contains_fn(hash));
     /// ```
-    #[cfg(not(feature = "no_index"))]
+    #[cfg(any(not(feature = "no_index"), not(feature = "no_object")))]
     #[inline(always)]
     pub fn set_indexer_set_fn<ARGS, A, B, C, F>(&mut self, func: F) -> u64
     where
@@ -1040,6 +1025,7 @@ impl Module {
         F: RegisterNativeFunction<ARGS, Result<(), Box<EvalAltResult>>>,
         F: Fn(&mut A, B, C) -> Result<(), Box<EvalAltResult>> + SendSync + 'static,
     {
+        #[cfg(not(feature = "no_index"))]
         if TypeId::of::<A>() == TypeId::of::<Array>() {
             panic!("Cannot register indexer for arrays.");
         }
@@ -1096,7 +1082,7 @@ impl Module {
     /// assert!(module.contains_fn(hash_get));
     /// assert!(module.contains_fn(hash_set));
     /// ```
-    #[cfg(not(feature = "no_index"))]
+    #[cfg(any(not(feature = "no_index"), not(feature = "no_object")))]
     #[inline(always)]
     pub fn set_indexer_get_set_fn<A, B, T>(
         &mut self,
@@ -1119,7 +1105,7 @@ impl Module {
     /// The [`u64`] hash is returned by the [`set_native_fn`][Module::set_native_fn] call.
     #[inline(always)]
     pub(crate) fn get_fn(&self, hash_fn: u64) -> Option<&CallableFunction> {
-        self.functions.get(&hash_fn).map(|f| &f.func)
+        self.functions.get(&hash_fn).map(|f| f.func.as_ref())
     }
 
     /// Does the particular namespace-qualified function exist in the [`Module`]?
@@ -1135,7 +1121,9 @@ impl Module {
     /// The [`u64`] hash is calculated by [`build_index`][Module::build_index].
     #[inline(always)]
     pub(crate) fn get_qualified_fn(&self, hash_qualified_fn: u64) -> Option<&CallableFunction> {
-        self.all_functions.get(&hash_qualified_fn)
+        self.all_functions
+            .get(&hash_qualified_fn)
+            .map(|f| f.as_ref())
     }
 
     /// Combine another [`Module`] into this [`Module`].
@@ -1426,7 +1414,10 @@ impl Module {
             match aliases.len() {
                 0 => (),
                 1 => {
-                    module.set_var(aliases.pop().unwrap(), value);
+                    let alias = aliases
+                        .pop()
+                        .expect("never fails because the list has one item");
+                    module.set_var(alias, value);
                 }
                 _ => aliases.into_iter().for_each(|alias| {
                     module.set_var(alias, value.clone());
@@ -1454,7 +1445,7 @@ impl Module {
             .filter(|f| f.func.is_script())
             .for_each(|f| {
                 // Encapsulate AST environment
-                let mut func = crate::fn_native::shared_take_or_clone(f.func.get_fn_def().clone());
+                let mut func = f.func.get_fn_def().as_ref().clone();
                 func.lib = Some(ast.shared_lib());
                 func.mods = func_mods.clone();
                 module.set_script_fn(func);
@@ -1486,7 +1477,7 @@ impl Module {
             module: &'a Module,
             path: &mut Vec<&'a str>,
             variables: &mut BTreeMap<u64, Dynamic>,
-            functions: &mut BTreeMap<u64, CallableFunction>,
+            functions: &mut BTreeMap<u64, Shared<CallableFunction>>,
             type_iterators: &mut BTreeMap<TypeId, IteratorFn>,
         ) -> bool {
             let mut contains_indexed_global_functions = false;
@@ -1502,7 +1493,7 @@ impl Module {
 
             // Index all variables
             module.variables.iter().for_each(|(var_name, value)| {
-                let hash_var = crate::calc_fn_hash(path.iter().map(|&v| v), var_name, 0);
+                let hash_var = crate::calc_qualified_fn_hash(path.iter().map(|&v| v), var_name, 0);
                 variables.insert(hash_var, value.clone());
             });
 
@@ -1532,8 +1523,11 @@ impl Module {
                         calc_native_fn_hash(path.iter().cloned(), f.name.as_str(), &f.param_types);
                     functions.insert(hash_qualified_fn, f.func.clone());
                 } else if cfg!(not(feature = "no_function")) {
-                    let hash_qualified_script =
-                        crate::calc_fn_hash(path.iter().cloned(), f.name.as_str(), f.params);
+                    let hash_qualified_script = crate::calc_qualified_fn_hash(
+                        path.iter().cloned(),
+                        f.name.as_str(),
+                        f.params,
+                    );
                     functions.insert(hash_qualified_script, f.func.clone());
                 }
             });
@@ -1690,6 +1684,8 @@ impl DerefMut for NamespaceRef {
 impl From<StaticVec<Ident>> for NamespaceRef {
     #[inline(always)]
     fn from(path: StaticVec<Ident>) -> Self {
+        let mut path = path;
+        path.shrink_to_fit();
         Self { index: None, path }
     }
 }
