@@ -4,7 +4,6 @@ use crate::ast::{Expr, OpAssignment, Stmt};
 use crate::dynamic::AccessMode;
 use crate::engine::{KEYWORD_DEBUG, KEYWORD_EVAL, KEYWORD_FN_PTR, KEYWORD_PRINT, KEYWORD_TYPE_OF};
 use crate::fn_builtin::get_builtin_binary_op_fn;
-use crate::parser::map_dynamic_to_expr;
 use crate::token::Token;
 use crate::utils::get_hasher;
 use crate::{
@@ -16,7 +15,6 @@ use std::prelude::v1::*;
 use std::{
     any::TypeId,
     hash::{Hash, Hasher},
-    iter::empty,
     mem,
 };
 
@@ -32,24 +30,14 @@ pub enum OptimizationLevel {
     Full,
 }
 
-impl OptimizationLevel {
-    /// Is the `OptimizationLevel` [`None`][OptimizationLevel::None]?
-    #[allow(dead_code)]
+impl Default for OptimizationLevel {
     #[inline(always)]
-    pub fn is_none(self) -> bool {
-        self == Self::None
-    }
-    /// Is the `OptimizationLevel` [`Simple`][OptimizationLevel::Simple]?
-    #[allow(dead_code)]
-    #[inline(always)]
-    pub fn is_simple(self) -> bool {
-        self == Self::Simple
-    }
-    /// Is the `OptimizationLevel` [`Full`][OptimizationLevel::Full]?
-    #[allow(dead_code)]
-    #[inline(always)]
-    pub fn is_full(self) -> bool {
-        self == Self::Full
+    fn default() -> Self {
+        if cfg!(feature = "no_optimize") {
+            Self::None
+        } else {
+            Self::Simple
+        }
     }
 }
 
@@ -59,7 +47,7 @@ struct State<'a> {
     /// Has the [`AST`] been changed during this pass?
     changed: bool,
     /// Collection of constants to use for eager function evaluations.
-    variables: Vec<(String, AccessMode, Expr)>,
+    variables: Vec<(String, AccessMode, Option<Dynamic>)>,
     /// Activate constants propagation?
     propagate_constants: bool,
     /// An [`Engine`] instance for eager function evaluation.
@@ -109,21 +97,21 @@ impl<'a> State<'a> {
     }
     /// Add a new constant to the list.
     #[inline(always)]
-    pub fn push_var(&mut self, name: &str, access: AccessMode, value: Expr) {
+    pub fn push_var(&mut self, name: &str, access: AccessMode, value: Option<Dynamic>) {
         self.variables.push((name.into(), access, value))
     }
     /// Look up a constant from the list.
     #[inline]
-    pub fn find_constant(&self, name: &str) -> Option<&Expr> {
+    pub fn find_constant(&self, name: &str) -> Option<&Dynamic> {
         if !self.propagate_constants {
             return None;
         }
 
-        self.variables.iter().rev().find_map(|(n, access, expr)| {
+        self.variables.iter().rev().find_map(|(n, access, value)| {
             if n == name {
                 match access {
                     AccessMode::ReadWrite => None,
-                    AccessMode::ReadOnly => Some(expr),
+                    AccessMode::ReadOnly => value.as_ref(),
                 }
             } else {
                 None
@@ -158,7 +146,7 @@ fn call_fn_with_constant_arguments(
             &mut Default::default(),
             state.lib,
             fn_name,
-            calc_fn_hash(empty(), fn_name, arg_values.len()),
+            calc_fn_hash(fn_name, arg_values.len()),
             arg_values.iter_mut().collect::<StaticVec<_>>().as_mut(),
             false,
             false,
@@ -217,13 +205,17 @@ fn optimize_stmt_block(
                     optimize_expr(value_expr, state);
 
                     if value_expr.is_constant() {
-                        state.push_var(&x.name, AccessMode::ReadOnly, value_expr.clone());
+                        state.push_var(
+                            &x.name,
+                            AccessMode::ReadOnly,
+                            value_expr.get_constant_value(),
+                        );
                     }
                 }
                 // Add variables into the state
                 Stmt::Let(value_expr, x, _, _) => {
                     optimize_expr(value_expr, state);
-                    state.push_var(&x.name, AccessMode::ReadWrite, Expr::Unit(x.pos));
+                    state.push_var(&x.name, AccessMode::ReadWrite, None);
                 }
                 // Optimize the statement
                 _ => optimize_stmt(stmt, state, preserve_result),
@@ -390,7 +382,7 @@ fn optimize_stmt(stmt: &mut Stmt, state: &mut State, preserve_result: bool) {
                     x.2 = if x2.args.len() > 1 {
                         mem::take(&mut x2.args[1])
                     } else {
-                        let (value, pos) = mem::take(&mut x2.constant_args[0]);
+                        let (value, pos) = mem::take(&mut x2.literal_args[0]);
                         Expr::DynamicConstant(Box::new(value), pos)
                     };
                 }
@@ -417,7 +409,7 @@ fn optimize_stmt(stmt: &mut Stmt, state: &mut State, preserve_result: bool) {
 
             *stmt = if preserve_result {
                 // -> { expr, Noop }
-                Stmt::Block(vec![Stmt::Expr(expr), Stmt::Noop(pos)], pos)
+                Stmt::Block(Box::new([Stmt::Expr(expr), Stmt::Noop(pos)]), pos)
             } else {
                 // -> expr
                 Stmt::Expr(expr)
@@ -434,7 +426,7 @@ fn optimize_stmt(stmt: &mut Stmt, state: &mut State, preserve_result: bool) {
             let else_block = mem::take(&mut *x.1).into_vec();
             *stmt = match optimize_stmt_block(else_block, state, preserve_result, true, false) {
                 statements if statements.is_empty() => Stmt::Noop(x.1.position()),
-                statements => Stmt::Block(statements, x.1.position()),
+                statements => Stmt::Block(statements.into_boxed_slice(), x.1.position()),
             }
         }
         // if true { if_block } else { else_block } -> if_block
@@ -443,7 +435,7 @@ fn optimize_stmt(stmt: &mut Stmt, state: &mut State, preserve_result: bool) {
             let if_block = mem::take(&mut *x.0).into_vec();
             *stmt = match optimize_stmt_block(if_block, state, preserve_result, true, false) {
                 statements if statements.is_empty() => Stmt::Noop(x.0.position()),
-                statements => Stmt::Block(statements, x.0.position()),
+                statements => Stmt::Block(statements.into_boxed_slice(), x.0.position()),
             }
         }
         // if expr { if_block } else { else_block }
@@ -484,7 +476,7 @@ fn optimize_stmt(stmt: &mut Stmt, state: &mut State, preserve_result: bool) {
                         condition,
                         Box::new((
                             mem::take(&mut block.1),
-                            Stmt::Block(def_stmt, def_pos).into(),
+                            Stmt::Block(def_stmt.into_boxed_slice(), def_pos).into(),
                         )),
                         match_expr.position(),
                     );
@@ -494,7 +486,7 @@ fn optimize_stmt(stmt: &mut Stmt, state: &mut State, preserve_result: bool) {
                     let statements = mem::take(&mut *block.1);
                     let statements =
                         optimize_stmt_block(statements.into_vec(), state, true, true, false);
-                    *stmt = Stmt::Block(statements, new_pos);
+                    *stmt = Stmt::Block(statements.into_boxed_slice(), new_pos);
                 }
             } else {
                 // Promote the default case
@@ -505,19 +497,20 @@ fn optimize_stmt(stmt: &mut Stmt, state: &mut State, preserve_result: bool) {
                 } else {
                     x.1.position()
                 };
-                *stmt = Stmt::Block(def_stmt, def_pos);
+                *stmt = Stmt::Block(def_stmt.into_boxed_slice(), def_pos);
             }
         }
         // switch
         Stmt::Switch(match_expr, x, _) => {
             optimize_expr(match_expr, state);
             x.0.values_mut().for_each(|block| {
-                let condition = if let Some(mut condition) = mem::take(&mut block.0) {
-                    optimize_expr(&mut condition, state);
-                    condition
-                } else {
-                    Expr::Unit(Position::NONE)
-                };
+                let condition = mem::take(&mut block.0).map_or_else(
+                    || Expr::Unit(Position::NONE),
+                    |mut condition| {
+                        optimize_expr(&mut condition, state);
+                        condition
+                    },
+                );
 
                 match condition {
                     Expr::Unit(_) | Expr::BoolConstant(true, _) => (),
@@ -572,7 +565,7 @@ fn optimize_stmt(stmt: &mut Stmt, state: &mut State, preserve_result: bool) {
                             if preserve_result {
                                 statements.push(Stmt::Noop(pos))
                             }
-                            *stmt = Stmt::Block(statements, pos);
+                            *stmt = Stmt::Block(statements.into_boxed_slice(), pos);
                         } else {
                             *stmt = Stmt::Noop(pos);
                         };
@@ -588,7 +581,7 @@ fn optimize_stmt(stmt: &mut Stmt, state: &mut State, preserve_result: bool) {
             let block_pos = body.position();
             let block = mem::take(body.statements()).into_vec();
             *stmt = Stmt::Block(
-                optimize_stmt_block(block, state, false, true, false),
+                optimize_stmt_block(block, state, false, true, false).into_boxed_slice(),
                 block_pos,
             );
         }
@@ -611,8 +604,8 @@ fn optimize_stmt(stmt: &mut Stmt, state: &mut State, preserve_result: bool) {
         Stmt::Import(expr, _, _) => optimize_expr(expr, state),
         // { block }
         Stmt::Block(statements, pos) => {
-            let mut block =
-                optimize_stmt_block(mem::take(statements), state, preserve_result, true, false);
+            let statements = mem::take(statements).into_vec();
+            let mut block = optimize_stmt_block(statements, state, preserve_result, true, false);
 
             match block.as_mut_slice() {
                 [] => {
@@ -624,7 +617,7 @@ fn optimize_stmt(stmt: &mut Stmt, state: &mut State, preserve_result: bool) {
                     state.set_dirty();
                     *stmt = mem::take(s);
                 }
-                _ => *stmt = Stmt::Block(block, *pos),
+                _ => *stmt = Stmt::Block(block.into_boxed_slice(), *pos),
             }
         }
         // try { pure try_block } catch ( var ) { catch_block } -> try_block
@@ -634,7 +627,7 @@ fn optimize_stmt(stmt: &mut Stmt, state: &mut State, preserve_result: bool) {
             let try_pos = x.0.position();
             let try_block = mem::take(&mut *x.0).into_vec();
             *stmt = Stmt::Block(
-                optimize_stmt_block(try_block, state, false, true, false),
+                optimize_stmt_block(try_block, state, false, true, false).into_boxed_slice(),
                 try_pos,
             );
         }
@@ -703,7 +696,7 @@ fn optimize_expr(expr: &mut Expr, state: &mut State) {
         Expr::Dot(x, _) => match (&mut x.lhs, &mut x.rhs) {
             // map.string
             (Expr::Map(m, pos), Expr::Property(p)) if m.0.iter().all(|(_, x)| x.is_pure()) => {
-                let prop = &p.2.name;
+                let prop = p.2.0.as_str();
                 // Map literal where everything is pure - promote the indexed item.
                 // All other items can be thrown away.
                 state.set_dirty();
@@ -820,6 +813,8 @@ fn optimize_expr(expr: &mut Expr, state: &mut State) {
                     }
                 }
             }
+
+            x.shrink_to_fit();
         }
         // [ constant .. ]
         #[cfg(not(feature = "no_index"))]
@@ -893,12 +888,12 @@ fn optimize_expr(expr: &mut Expr, state: &mut State) {
             if !x.is_qualified() // Non-qualified
             && state.optimization_level == OptimizationLevel::Simple // simple optimizations
             && x.args_count() == 1
-            && x.constant_args.len() == 1
-            && x.constant_args[0].0.is::<ImmutableString>()
+            && x.literal_args.len() == 1
+            && x.literal_args[0].0.is::<ImmutableString>()
             && x.name == KEYWORD_FN_PTR
         => {
             state.set_dirty();
-            let fn_ptr = FnPtr::new_unchecked(mem::take(&mut x.constant_args[0].0).as_str_ref().unwrap().into(), Default::default());
+            let fn_ptr = FnPtr::new_unchecked(mem::take(&mut x.literal_args[0].0).as_str_ref().unwrap().into(), Default::default());
             *expr = Expr::DynamicConstant(Box::new(fn_ptr.into()), *pos);
         }
 
@@ -916,22 +911,23 @@ fn optimize_expr(expr: &mut Expr, state: &mut State) {
                 //&& !is_valid_identifier(x.name.chars()) // cannot be scripted
         => {
             let mut arg_values: StaticVec<_> = x.args.iter().map(|e| e.get_constant_value().unwrap())
-                                                .chain(x.constant_args.iter().map(|(v, _)| v).cloned())
+                                                .chain(x.literal_args.iter().map(|(v, _)| v).cloned())
                                                 .collect();
 
             let arg_types: StaticVec<_> = arg_values.iter().map(Dynamic::type_id).collect();
 
             // Search for overloaded operators (can override built-in).
             if !has_native_fn(state, x.hashes.native_hash(), arg_types.as_ref()) {
-                if let Some(result) = get_builtin_binary_op_fn(x.name.as_ref(), &arg_values[0], &arg_values[1])
-                                        .and_then(|f| {
-                                            let ctx = (state.engine, x.name.as_ref(), state.lib).into();
-                                            let (first, second) = arg_values.split_first_mut().unwrap();
-                                            (f)(ctx, &mut [ first, &mut second[0] ]).ok()
-                                        })
-                                        .and_then(|result| map_dynamic_to_expr(result, *pos))
+                if let Some(mut result) = get_builtin_binary_op_fn(x.name.as_ref(), &arg_values[0], &arg_values[1])
+                                            .and_then(|f| {
+                                                let ctx = (state.engine, x.name.as_ref(), state.lib).into();
+                                                let (first, second) = arg_values.split_first_mut().unwrap();
+                                                (f)(ctx, &mut [ first, &mut second[0] ]).ok()
+                                            })
+                                            .map(Expr::from)
                 {
                     state.set_dirty();
+                    result.set_position(*pos);
                     *expr = result;
                     return;
                 }
@@ -943,10 +939,11 @@ fn optimize_expr(expr: &mut Expr, state: &mut State) {
             while x.args.last().map(Expr::is_constant).unwrap_or(false) {
                 let arg = x.args.pop().unwrap();
                 let arg_pos = arg.position();
-                x.constant_args.insert(0, (arg.get_constant_value().unwrap(), arg_pos));
+                x.literal_args.insert(0, (arg.get_constant_value().unwrap(), arg_pos));
             }
 
             x.args.shrink_to_fit();
+            x.literal_args.shrink_to_fit();
         }
 
         // Eagerly call functions
@@ -963,7 +960,7 @@ fn optimize_expr(expr: &mut Expr, state: &mut State) {
 
             if !has_script_fn {
                 let mut arg_values: StaticVec<_> = x.args.iter().map(|e| e.get_constant_value().unwrap())
-                                                    .chain(x.constant_args.iter().map(|(v, _)| v).cloned())
+                                                    .chain(x.literal_args.iter().map(|(v, _)| v).cloned())
                                                     .collect();
 
                 // Save the typename of the first argument if it is `type_of()`
@@ -974,18 +971,19 @@ fn optimize_expr(expr: &mut Expr, state: &mut State) {
                     ""
                 };
 
-                if let Some(result) = call_fn_with_constant_arguments(&state, x.name.as_ref(), &mut arg_values)
-                                        .or_else(|| {
-                                            if !arg_for_type_of.is_empty() {
-                                                // Handle `type_of()`
-                                                Some(arg_for_type_of.to_string().into())
-                                            } else {
-                                                None
-                                            }
-                                        })
-                                        .and_then(|result| map_dynamic_to_expr(result, *pos))
+                if let Some(mut result) = call_fn_with_constant_arguments(&state, x.name.as_ref(), &mut arg_values)
+                                            .or_else(|| {
+                                                if !arg_for_type_of.is_empty() {
+                                                    // Handle `type_of()`
+                                                    Some(arg_for_type_of.to_string().into())
+                                                } else {
+                                                    None
+                                                }
+                                            })
+                                            .map(Expr::from)
                 {
                     state.set_dirty();
+                    result.set_position(*pos);
                     *expr = result;
                     return;
                 }
@@ -1002,25 +1000,25 @@ fn optimize_expr(expr: &mut Expr, state: &mut State) {
             while x.args.last().map(Expr::is_constant).unwrap_or(false) {
                 let arg = x.args.pop().unwrap();
                 let arg_pos = arg.position();
-                x.constant_args.insert(0, (arg.get_constant_value().unwrap(), arg_pos));
+                x.literal_args.insert(0, (arg.get_constant_value().unwrap(), arg_pos));
             }
 
             x.args.shrink_to_fit();
+            x.literal_args.shrink_to_fit();
         }
 
         // constant-name
         Expr::Variable(_, pos, x) if x.1.is_none() && state.find_constant(&x.2).is_some() => {
-            state.set_dirty();
-
             // Replace constant with value
-            let mut result = state.find_constant(&x.2).unwrap().clone();
-            result.set_position(*pos);
-            *expr = result;
+            let pos = *pos;
+            *expr = Expr::from(state.find_constant(&x.2).unwrap().clone());
+            expr.set_position(pos);
+            state.set_dirty();
         }
 
         // Custom syntax
         Expr::Custom(x, _) => {
-            if x.scope_delta != 0 {
+            if x.scope_changed {
                 state.propagate_constants = false;
             }
             x.keywords.iter_mut().for_each(|expr| optimize_expr(expr, state));
@@ -1051,11 +1049,9 @@ fn optimize_top_level(
     // Add constants and variables from the scope
     scope.iter().for_each(|(name, constant, value)| {
         if !constant {
-            state.push_var(name, AccessMode::ReadWrite, Expr::Unit(Position::NONE));
-        } else if let Some(val) = map_dynamic_to_expr(value, Position::NONE) {
-            state.push_var(name, AccessMode::ReadOnly, val);
+            state.push_var(name, AccessMode::ReadWrite, None);
         } else {
-            state.push_var(name, AccessMode::ReadOnly, Expr::Unit(Position::NONE));
+            state.push_var(name, AccessMode::ReadOnly, Some(value));
         }
     });
 
@@ -1072,7 +1068,7 @@ pub fn optimize_into_ast(
     optimization_level: OptimizationLevel,
 ) -> AST {
     let level = if cfg!(feature = "no_optimize") {
-        OptimizationLevel::None
+        Default::default()
     } else {
         optimization_level
     };
@@ -1081,7 +1077,7 @@ pub fn optimize_into_ast(
     let lib = {
         let mut module = Module::new();
 
-        if !level.is_none() {
+        if level != OptimizationLevel::None {
             // We only need the script library's signatures for optimization purposes
             let mut lib2 = Module::new();
 
