@@ -6,6 +6,7 @@ pub(crate) mod arithmetic;
 mod array_basic;
 mod fn_basic;
 mod iter_basic;
+mod lang_core;
 mod logic;
 mod map_basic;
 mod math_basic;
@@ -86,6 +87,7 @@ macro_rules! def_package {
         }
 
         impl $package {
+            #[allow(dead_code)]
             pub fn new() -> Self {
                 let mut module = $root::Module::new();
                 <Self as $root::packages::Package>::init(&mut module);
