@@ -973,10 +973,6 @@ pub struct TokenizeState {
     pub comment_level: usize,
     /// Include comments?
     pub include_comments: bool,
-    /// Disable doc-comments?
-    #[cfg(not(feature = "no_function"))]
-    #[cfg(feature = "metadata")]
-    pub disable_doc_comments: bool,
     /// Is the current tokenizer position within the text stream of an interpolated string?
     pub is_within_text_terminated_by: Option<char>,
 }
@@ -1185,7 +1181,10 @@ pub fn parse_string_literal(
 
                 #[cfg(not(feature = "no_position"))]
                 {
-                    skip_whitespace_until = start.position().unwrap() + 1;
+                    let start_position = start
+                        .position()
+                        .expect("never fails because the string must have a starting position");
+                    skip_whitespace_until = start_position + 1;
                 }
             }
 
@@ -1205,7 +1204,11 @@ pub fn parse_string_literal(
 
             // Whitespace to skip
             #[cfg(not(feature = "no_position"))]
-            _ if next_char.is_whitespace() && pos.position().unwrap() < skip_whitespace_until => {}
+            _ if next_char.is_whitespace()
+                && pos
+                    .position()
+                    .expect("never fails because a character must have a position")
+                    < skip_whitespace_until => {}
 
             // All other characters
             _ => {
@@ -1241,37 +1244,29 @@ fn scan_block_comment(
     stream: &mut impl InputStream,
     mut level: usize,
     pos: &mut Position,
-    comment: &mut Option<String>,
+    mut comment: Option<&mut String>,
 ) -> usize {
+    let comment = &mut comment;
+
     while let Some(c) = stream.get_next() {
         pos.advance();
 
-        if let Some(ref mut comment) = comment {
-            comment.push(c);
-        }
+        comment.as_mut().map(|comment| comment.push(c));
 
         match c {
             '/' => {
-                if let Some(c2) = stream.peek_next() {
-                    if c2 == '*' {
-                        eat_next(stream, pos);
-                        if let Some(ref mut comment) = comment {
-                            comment.push(c2);
-                        }
-                        level += 1;
-                    }
-                }
+                stream.peek_next().filter(|&c2| c2 == '*').map(|c2| {
+                    eat_next(stream, pos);
+                    comment.as_mut().map(|comment| comment.push(c2));
+                    level += 1;
+                });
             }
             '*' => {
-                if let Some(c2) = stream.peek_next() {
-                    if c2 == '/' {
-                        eat_next(stream, pos);
-                        if let Some(ref mut comment) = comment {
-                            comment.push(c2);
-                        }
-                        level -= 1;
-                    }
-                }
+                stream.peek_next().filter(|&c2| c2 == '/').map(|c2| {
+                    eat_next(stream, pos);
+                    comment.as_mut().map(|comment| comment.push(c2));
+                    level -= 1;
+                });
             }
             '\n' => pos.new_line(),
             _ => (),
@@ -1351,22 +1346,27 @@ fn get_next_token_inner(
             None
         };
 
-        state.comment_level = scan_block_comment(stream, state.comment_level, pos, &mut comment);
+        state.comment_level =
+            scan_block_comment(stream, state.comment_level, pos, comment.as_mut());
 
-        let include_comments = state.include_comments;
+        let return_comment = state.include_comments;
 
         #[cfg(not(feature = "no_function"))]
         #[cfg(feature = "metadata")]
-        let include_comments =
-            if !state.disable_doc_comments && is_doc_comment(comment.as_ref().unwrap()) {
-                true
-            } else {
-                include_comments
-            };
+        let return_comment = return_comment
+            || is_doc_comment(
+                comment
+                    .as_ref()
+                    .expect("never fails because `include_comments` is true"),
+            );
 
-        if include_comments {
-            return Some((Token::Comment(comment.unwrap()), start_pos));
-        } else if state.comment_level > 0 {
+        if return_comment {
+            return Some((
+                Token::Comment(comment.expect("never fails because `return_comment` is true")),
+                start_pos,
+            ));
+        }
+        if state.comment_level > 0 {
             // Reached EOF without ending comment block
             return None;
         }
@@ -1414,7 +1414,7 @@ fn get_next_token_inner(
                         }
                         #[cfg(any(not(feature = "no_float"), feature = "decimal"))]
                         '.' => {
-                            stream.get_next().unwrap();
+                            stream.get_next().expect("never fails because it is `.`");
 
                             // Check if followed by digits or something that cannot start a property name
                             match stream.peek_next().unwrap_or('\0') {
@@ -1448,7 +1448,7 @@ fn get_next_token_inner(
                         }
                         #[cfg(not(feature = "no_float"))]
                         'e' => {
-                            stream.get_next().unwrap();
+                            stream.get_next().expect("never fails it is `e`");
 
                             // Check if followed by digits or +/-
                             match stream.peek_next().unwrap_or('\0') {
@@ -1461,7 +1461,11 @@ fn get_next_token_inner(
                                 '+' | '-' => {
                                     result.push(next_char);
                                     pos.advance();
-                                    result.push(stream.get_next().unwrap());
+                                    result.push(
+                                        stream
+                                            .get_next()
+                                            .expect("never fails because it is `+` or `-`"),
+                                    );
                                     pos.advance();
                                 }
                                 // Not a floating-point number
@@ -1497,12 +1501,10 @@ fn get_next_token_inner(
                     }
                 }
 
-                let num_pos = if let Some(negated_pos) = negated {
+                let num_pos = negated.map_or(start_pos, |negated_pos| {
                     result.insert(0, '-');
                     negated_pos
-                } else {
-                    start_pos
-                };
+                });
 
                 // Parse number
                 return Some((
@@ -1705,7 +1707,7 @@ fn get_next_token_inner(
                 let mut comment = match stream.peek_next() {
                     #[cfg(not(feature = "no_function"))]
                     #[cfg(feature = "metadata")]
-                    Some('/') if !state.disable_doc_comments => {
+                    Some('/') => {
                         eat_next(stream, pos);
 
                         // Long streams of `///...` are not doc-comments
@@ -1723,9 +1725,7 @@ fn get_next_token_inner(
                         pos.new_line();
                         break;
                     }
-                    if let Some(ref mut comment) = comment {
-                        comment.push(c);
-                    }
+                    comment.as_mut().map(|comment| comment.push(c));
                     pos.advance();
                 }
 
@@ -1740,7 +1740,7 @@ fn get_next_token_inner(
                 let mut comment = match stream.peek_next() {
                     #[cfg(not(feature = "no_function"))]
                     #[cfg(feature = "metadata")]
-                    Some('*') if !state.disable_doc_comments => {
+                    Some('*') => {
                         eat_next(stream, pos);
 
                         // Long streams of `/****...` are not doc-comments
@@ -1754,7 +1754,7 @@ fn get_next_token_inner(
                 };
 
                 state.comment_level =
-                    scan_block_comment(stream, state.comment_level, pos, &mut comment);
+                    scan_block_comment(stream, state.comment_level, pos, comment.as_mut());
 
                 if let Some(comment) = comment {
                     return Some((Token::Comment(comment), start_pos));
@@ -2184,8 +2184,8 @@ impl<'a> Iterator for TokenIterator<'a> {
         };
 
         // Run the mapper, if any
-        let token = if let Some(map) = self.map {
-            map(token)
+        let token = if let Some(map_func) = self.map {
+            map_func(token)
         } else {
             token
         };
@@ -2239,9 +2239,6 @@ impl Engine {
                     non_unary: false,
                     comment_level: 0,
                     include_comments: false,
-                    #[cfg(not(feature = "no_function"))]
-                    #[cfg(feature = "metadata")]
-                    disable_doc_comments: self.disable_doc_comments,
                     is_within_text_terminated_by: None,
                 },
                 pos: Position::new(1, 0),
