@@ -10,7 +10,7 @@ use std::{
     cmp::Ordering,
     fmt,
     hash::{BuildHasher, Hash, Hasher},
-    iter::FromIterator,
+    iter::{empty, FromIterator},
     ops::{Add, AddAssign, Deref, Sub, SubAssign},
     str::FromStr,
 };
@@ -58,9 +58,8 @@ pub fn get_hasher() -> ahash::AHasher {
     Default::default()
 }
 
-/// _(INTERNALS)_ Calculate a [`u64`] hash key from a namespace-qualified function name
+/// Calculate a [`u64`] hash key from a namespace-qualified function name
 /// and the number of parameters, but no parameter types.
-/// Exported under the `internals` feature only.
 ///
 /// Module names are passed in via `&str` references from an iterator.
 /// Parameter types are passed in via [`TypeId`] values from an iterator.
@@ -68,8 +67,8 @@ pub fn get_hasher() -> ahash::AHasher {
 /// # Note
 ///
 /// The first module name is skipped.  Hashing starts from the _second_ module in the chain.
-#[inline(always)]
-pub fn calc_fn_hash<'a>(
+#[inline]
+pub fn calc_qualified_fn_hash<'a>(
     modules: impl Iterator<Item = &'a str>,
     fn_name: impl AsRef<str>,
     num: usize,
@@ -88,11 +87,19 @@ pub fn calc_fn_hash<'a>(
     s.finish()
 }
 
-/// _(INTERNALS)_ Calculate a [`u64`] hash key from a list of parameter types.
-/// Exported under the `internals` feature only.
+/// Calculate a [`u64`] hash key from a non-namespace-qualified function name
+/// and the number of parameters, but no parameter types.
 ///
 /// Parameter types are passed in via [`TypeId`] values from an iterator.
 #[inline(always)]
+pub fn calc_fn_hash(fn_name: impl AsRef<str>, num: usize) -> u64 {
+    calc_qualified_fn_hash(empty(), fn_name, num)
+}
+
+/// Calculate a [`u64`] hash key from a list of parameter types.
+///
+/// Parameter types are passed in via [`TypeId`] values from an iterator.
+#[inline]
 pub fn calc_fn_params_hash(params: impl Iterator<Item = TypeId>) -> u64 {
     let s = &mut get_hasher();
     let mut len = 0;
@@ -198,6 +205,7 @@ impl From<String> for ImmutableString {
         Self(Into::<SmartString>::into(value).into())
     }
 }
+#[cfg(not(feature = "no_smartstring"))]
 impl From<SmartString> for ImmutableString {
     #[inline(always)]
     fn from(value: SmartString) -> Self {
@@ -248,6 +256,14 @@ impl<'a> FromIterator<String> for ImmutableString {
     }
 }
 
+#[cfg(not(feature = "no_smartstring"))]
+impl<'a> FromIterator<SmartString> for ImmutableString {
+    #[inline(always)]
+    fn from_iter<T: IntoIterator<Item = SmartString>>(iter: T) -> Self {
+        Self(iter.into_iter().collect::<SmartString>().into())
+    }
+}
+
 impl fmt::Display for ImmutableString {
     #[inline(always)]
     fn fmt(&self, f: &mut fmt::Formatter<'_>) -> fmt::Result {
@@ -618,17 +634,17 @@ impl ImmutableString {
 /// yet interned.
 #[derive(Debug, Clone, Default, Hash)]
 pub struct IdentifierBuilder(
-    #[cfg(feature = "no_smartstring_for_identifier")] std::collections::BTreeSet<Identifier>,
+    #[cfg(feature = "no_smartstring")] std::collections::BTreeSet<Identifier>,
 );
 
 impl IdentifierBuilder {
     /// Get an identifier from a text string.
     #[inline(always)]
     pub fn get(&mut self, text: impl AsRef<str> + Into<Identifier>) -> Identifier {
-        #[cfg(not(feature = "no_smartstring_for_identifier"))]
+        #[cfg(not(feature = "no_smartstring"))]
         return text.as_ref().into();
 
-        #[cfg(feature = "no_smartstring_for_identifier")]
+        #[cfg(feature = "no_smartstring")]
         return self.0.get(text.as_ref()).cloned().unwrap_or_else(|| {
             let s: Identifier = text.into();
             self.0.insert(s.clone());
